From 6c3d33fad62c0afd5df83482b0d859a72dce665f Mon Sep 17 00:00:00 2001
From: Dirk Mueller <dmueller@suse.com>
Date: Sun, 15 Mar 2015 12:44:37 +0100
Subject: [PATCH] ARM64: Fix support for ARMv7 userspace

---
 patches.arch/arm64-fix-mmap2-compat-syscall.patch | 70 +++++++++++++++++++++++
 series.conf                                       |  1 +
 2 files changed, 71 insertions(+)
 create mode 100644 patches.arch/arm64-fix-mmap2-compat-syscall.patch

diff --git a/patches.arch/arm64-fix-mmap2-compat-syscall.patch b/patches.arch/arm64-fix-mmap2-compat-syscall.patch
new file mode 100644
index 0000000..d35db07
--- /dev/null
+++ b/patches.arch/arm64-fix-mmap2-compat-syscall.patch
@@ -0,0 +1,70 @@
+From: Andreas Schwab <schwab@suse.de>
+Date: Mon, 9 Mar 2015 17:27:36 +0100
+Subject: [PATCH] arm64: fix implementation of mmap2 compat syscall
+Patch-mainline: Submitted, http://lists.infradead.org/pipermail/linux-arm-kernel/2015-March/330081.html
+References: bnc#923505
+
+The arm mmap2 syscall takes the offset in units of 4K, thus with 64K pages
+the offset needs to be scaled to units of pages.
+
+Tested-by: Alexander Graf <agraf@suse.de>
+Signed-off-by: Andreas Schwab <schwab@suse.de>
+[dmueller: backport to stable]
+Signed-off-by: Dirk Mueller <dmueller@suse.com>
+
+---
+ arch/arm64/include/asm/unistd32.h |  2 +-
+ arch/arm64/kernel/sys32.S         | 17 +++++++++++++++++
+ 2 files changed, 18 insertions(+), 1 deletion(-)
+
+Index: current/arch/arm64/include/asm/unistd32.h
+===================================================================
+--- current.orig/arch/arm64/include/asm/unistd32.h
++++ current/arch/arm64/include/asm/unistd32.h
+@@ -406,7 +406,7 @@ __SYSCALL(__NR_vfork, sys_vfork)
+ #define __NR_ugetrlimit 191	/* SuS compliant getrlimit */
+ __SYSCALL(__NR_ugetrlimit, compat_sys_getrlimit)		/* SuS compliant getrlimit */
+ #define __NR_mmap2 192
+-__SYSCALL(__NR_mmap2, sys_mmap_pgoff)
++__SYSCALL(__NR_mmap2, compat_sys_mmap2_wrapper)
+ #define __NR_truncate64 193
+ __SYSCALL(__NR_truncate64, compat_sys_truncate64_wrapper)
+ #define __NR_ftruncate64 194
+Index: current/arch/arm64/kernel/sys32.S
+===================================================================
+--- current.orig/arch/arm64/kernel/sys32.S
++++ current/arch/arm64/kernel/sys32.S
+@@ -19,9 +19,12 @@
+  */
+ 
+ #include <linux/linkage.h>
++#include <linux/const.h>
+ 
+ #include <asm/assembler.h>
+ #include <asm/asm-offsets.h>
++#include <asm/errno.h>
++#include <asm/page.h>
+ 
+ /*
+  * System call wrappers for the AArch32 compatibility layer.
+@@ -53,6 +56,20 @@ compat_sys_fstatfs64_wrapper:
+ 	b	compat_sys_fstatfs64
+ ENDPROC(compat_sys_fstatfs64_wrapper)
+ 
++
++
++/*
++ * Note: off_4k (w5) is always units of 4K.  If we can't do the requested
++ * offset, we return EINVAL.
++ */
++compat_sys_mmap2_wrapper:
++       tst     w5, #~PAGE_MASK >> 12
++       b.ne    1f
++       lsr     w5, w5, #PAGE_SHIFT - 12
++       b       sys_mmap_pgoff
++1:     mov     x0, #-EINVAL
++       ret     lr
++
+ /*
+  * Wrappers for AArch32 syscalls that either take 64-bit parameters
+  * in registers or that take 32-bit parameters which require sign
diff --git a/series.conf b/series.conf
index 03e1a14..f31cfe8 100644
--- a/series.conf
+++ b/series.conf
@@ -174,6 +174,7 @@
 	patches.arch/arm64-0010-KVM-ARM-Hack-to-enable-VGIC-mapping-on-64k-PAGE_SIZE.patch
 	patches.arch/arm64-0011-Enable-CONFIG_COMPAT-also-for-64k-page-size.patch
 	patches.arch/arm64-0012-generic-pci.patch
+	patches.arch/arm64-fix-mmap2-compat-syscall.patch
 
 	########################################################
 	# S/390
-- 
2.0.4

